#import datetime
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>Hello from Jinja2</title>
  </head>
  <body>
    <h1>Hello from Jinja2</h1>
    <p>You visited the URL <tt>{{ h.url_for() }}</tt> at <tt>{{ c.now() }}</tt></p>
    <p>{{ c.test }}</p>
   </body>
</html>
