/* @override http://localhost:5000/_debug/media/pylons/style/itraceback.css */

div.credits {
    padding: 5px 0 0 0;
    color: #777;
    font-size: 11px;
}

#footer {
    border-top: 1px solid #666666;
    padding: 0;
    margin: 20px 0 0 0;
}

#pylons_logo {
    float: right;
    margin: 5px 0 0 0;
}

div.widget_layout {
    margin: 30px 0 0 0;
}
div.feature-highlight {
    float: right;
    width: 400px;
}

div.feature-highlight button {
    margin: 15px 0 0 20px;
}

div#service_widget {
    width: 500px;
    padding: 0;
    margin: 0;
    margin-right: 10px;
    line-height: 1.4;
}

div#service_widget h4, div.overviewtab h3 {
    margin: 0;
    padding: 0;
    font-weight: bold;
    font-size: 100%;
}

div#service_widget h2 {
    color: #000;
    border-bottom: 1px solid #0a0;
    font-variant: small-caps;
    font-size: 20px;
    line-height: 1.3;
    font-weight: bold;
}

div.overviewtab, div.posttracebacktab, div.searchtab {
    padding: 13px 10px 10px 10px;
    background-color: #f8f8f8;
    border-bottom: 1px solid #ccc;
    border-right: 1px solid #ccc;
    border-left: 1px solid #ccc;
}

div.clearfix {
    clear: left;
    margin: 0;
    padding: 0;
    height: 0;
}

.hide {
    display: none;
}

input.query {
    width: 350px;
}

p.query {
	margin: 0 0 0 20px;
	padding: 0;
}

/* Nav Bar */

#nv li {
    display:block; 
    float:left; 
    list-style-type:none; 
    margin:0; 
    padding:0;
}
#nv a { 
    display:block; 
    float:left; 
    border: 0px;
}




/*** Tabs ***/
#supportnav { float: left; width: 500px; border-bottom: 1px solid #CCC; margin: 0px; padding: 0px; }
    #supportnav li { float: left; padding-left: 10px; margin-right: 3px; background: url("../img/tab-traceback.png") no-repeat left top; }
    #supportnav li.first-child { margin: 0; }
    #supportnav li a { display: block; padding: 3px 10px 3px 0; border: none; outline: none; background: url("../img/tab-traceback.png") no-repeat right top; color: #666; }
    #supportnav li a:hover { color: #333; }
    #supportnav li.active { position: relative; margin-bottom: -1px; background-position: 0 -40px; }
    #supportnav li.active a { display: block; padding: 3px 10px 4px 0; background: url("../img/tab-traceback.png") no-repeat right -40px; font-weight: bold; color: #333; }
    #supportnav li.active a { background-position: right -40px; }
* html #supportnav { margin-bottom: 0; }
    
div.searchresults div.result {
	margin-bottom: 5px;
	padding: 2px 0px 2px 4px;
	cursor: pointer;
	background-color: rgb(247, 247, 247);
	border: 1px solid rgb(235, 235, 235);
}

div.searchresults div.result div.meta {
	font-size: 0.8em;
	color: rgb(1, 116, 22);
}

p.results a {
    float: right;
}

div.searchresults div.blurb {
	font-size: 0.8em;
}

div.searchresults div.result a {
	text-decoration: underline;
	font-size: 0.9em;
	cursor: pointer;
}
