body {
    font-family: arial, helvetica, sans-serif;
    background-color: #ffc900;
    background-image: url(../img/bg.png);
    background-repeat: repeat-x;
    width:100%;
    height:100%;
    margin:0;
    max-height: 100%;
    padding:0;
    border:none;
    line-height:1.4;
}
#container {
    color:white;
    background-color:#111;
    position: absolute;
    left: 50%;
    width: 500px;
    margin-left: -300px;
    padding:50px;
    height:100%;
}
#footer {
    margin: 120px 0 0 0;
    padding: 3px;
    text-align:center;
    font-size:small;
    background-color:#222;
    letter-spacing: 1px;
}
h1 {
    text-align:center;
    font-size:xx-large;
    font-weight:normal;
    margin: 0 0 20px 0;
    border:none;
    padding:0;
    letter-spacing: 5px;
}
h2 {
    font-size:xx-large;
    font-weight:normal;
    margin: 0 0 20px 0;
    border:none;
    padding:0;
}
hr {
    margin-bottom:30px;
    border: 1px solid #222;
    background-color: #222;
    padding: 2px;
}
#logo {
    background-image: url(../img/pylons-logo.gif);
    background-repeat: no-repeat;
    height: 0;
    overflow: hidden;
    padding-top: 99px;
    width: 239px;
}
#left {
    float:left;
    width:250px;
    margin:0 50px 0 0;
    border:none;
    padding:0 0 0 10px;
}
#right {
    margin:0 0 0 330px;
    border:none;
    padding:0;
}
ul {
    list-style:none;
    margin:0;
    border:none;
    padding:0;
}
a:visited {
    color:white;
    text-decoration:none;
}
a:link {
    color:white;
    text-decoration:none;
}
